@@ -108,7 +108,7 @@ class WPSRequest(object):
             try:
                 doc = etree.fromstring(self.http_request.get_data())
             except Exception as e:
-                raise NoApplicableCode(e.msg)
+                raise NoApplicableCode(str(e))
             operation = doc.tag
             version = get_version_from_ns(doc.nsmap[doc.prefix])
             self.set_version(version)
@@ -122,7 +122,7 @@ class WPSRequest(object):
             try:
                 jdoc = json.loads(self.http_request.get_data())
             except Exception as e:
-                raise NoApplicableCode(e.msg)
+                raise NoApplicableCode(str(e))
             if self.identifier is not None:
                 jdoc = {'inputs': jdoc}
             else:
@@ -491,8 +491,13 @@ class WPSRequest(object):
                     inpt_def = {"data": inpt_def}
                 if 'identifier' not in inpt_def:
                     inpt_def['identifier'] = identifier
-                inpt = input_from_json(inpt_def)
-                self.inputs[identifier].append(inpt)
+                try:
+                    inpt = input_from_json(inpt_def)
+                    self.inputs[identifier].append(inpt)
+                except Exception as e:
+                    LOGGER.warning(e)
+                    LOGGER.warning(f'skipping input: {identifier}')
+                    pass
 
 
 def get_inputs_from_xml(doc):
