@@ -5,17 +5,19 @@
 """
 WPS Output classes
 """
+from typing import Optional, Sequence, Dict, Union
 
-import lxml.etree as etree
+from pywps import xml_util as etree
 import os
 import re
 from pywps.app.Common import Metadata
 from pywps.exceptions import InvalidParameterValue
 from pywps.inout import basic
 from pywps.inout.storage.file import FileStorageBuilder
+from pywps.inout.types import Translations
 from pywps.validator.mode import MODE
 from pywps import configuration as config
-from pywps.inout.formats import Format
+from pywps.inout.formats import Format, Supported_Formats, FORMATS
 
 
 class BoundingBoxOutput(basic.BBoxOutput):
@@ -109,9 +111,10 @@ class ComplexOutput(basic.ComplexOutput):
         e.g. {"fr-CA": {"title": "Mon titre", "abstract": "Une description"}}
     """
 
-    def __init__(self, identifier, title, supported_formats=None,
-                 data_format=None, abstract='', keywords=[], workdir=None, metadata=None,
-                 as_reference=False, mode=MODE.NONE, translations=None):
+    def __init__(self, identifier: str, title: str, supported_formats: Supported_Formats = None,
+                 data_format=None, abstract: str = '', keywords=[], workdir=None,
+                 metadata: Optional[Sequence[Metadata]] = None,
+                 as_reference=False, mode: MODE = MODE.NONE, translations: Translations = None):
         if metadata is None:
             metadata = []
 
@@ -210,14 +213,12 @@ class ComplexOutput(basic.ComplexOutput):
     def _json_data(self, data):
         """Return Data node
         """
-        # Match only data that are safe CDATA pattern.
-        CDATA_PATTERN = re.compile(r'^<!\[CDATA\[((?!\]\]>).)*\]\]>$')
-
         data["type"] = "complex"
 
         if self.data:
-
-            if self.data_format.mime_type in ["application/xml", "application/gml+xml", "text/xml"]:
+            if self.data_format.mime_type in [FORMATS.GEOJSON.mime_type, FORMATS.JSON.mime_type]:
+                data["data"] = self.data
+            elif self.data_format.mime_type in ["application/xml", "application/gml+xml", "text/xml"]:
                 # Note that in a client-server round trip, the original and returned file will not be identical.
                 data_doc = etree.parse(self.file)
                 data["data"] = etree.tostring(data_doc, pretty_print=True).decode('utf-8')
@@ -226,6 +227,9 @@ class ComplexOutput(basic.ComplexOutput):
                 if self.data_format.encoding == 'base64':
                     data["data"] = self.base64.decode('utf-8')
                 else:
+                    # Match only data that are safe CDATA pattern.
+                    CDATA_PATTERN = re.compile(r'^<!\[CDATA\[((?!\]\]>).)*\]\]>$')
+
                     # Otherwise we assume all other formats are unsafe and need to be enclosed in a CDATA tag.
                     if isinstance(self.data, bytes):
                         # Try to inline data as text but if fail encode is in base64
@@ -539,13 +543,14 @@ class MetaLink4(MetaLink):
 
 
 def output_from_json(json_data):
-    if json_data['type'] == 'complex':
+    data_type = json_data.get('type', 'literal')
+    if data_type == 'complex':
         output = ComplexOutput.from_json(json_data)
-    elif json_data['type'] == 'literal':
+    elif data_type == 'literal':
         output = LiteralOutput.from_json(json_data)
-    elif json_data['type'] == 'bbox':
+    elif data_type == 'bbox':
         output = BoundingBoxOutput.from_json(json_data)
     else:
-        raise InvalidParameterValue("Output type not recognized: {}".format(json_data['type']))
+        raise InvalidParameterValue("Output type not recognized: {}".format(data_type))
 
     return output
