@@ -7,11 +7,10 @@
 # licensed under MIT, Please consult LICENSE.txt for details     #
 ##################################################################
 
-from __future__ import absolute_import
+
 import requests
 import os
 import tempfile
-import datetime
 import unittest
 import json
 from pywps import inout
@@ -22,19 +21,19 @@ from pywps.app.Common import Metadata
 from pywps.validator import get_validator
 from pywps.inout.basic import IOHandler, SOURCE_TYPE, SimpleHandler, BBoxInput, BBoxOutput, \
     ComplexInput, ComplexOutput, LiteralOutput, LiteralInput, _is_textfile
+from pywps.util import uri_to_path
 from pywps.inout.literaltypes import convert, AllowedValue, AnyValue
 from pywps.inout.outputs import MetaFile, MetaLink, MetaLink4
-from pywps._compat import StringIO, text_type, urlparse
+from io import StringIO
+from urllib.parse import urlparse
 from pywps.validator.base import emptyvalidator
 from pywps.exceptions import InvalidParameterValue
 from pywps.validator.mode import MODE
 from pywps.inout.basic import UOM
-from pywps._compat import PY2
 from pywps.inout.storage.file import FileStorageBuilder
 from pywps.tests import service_ok
 from pywps.translations import get_translation
 
-from lxml import etree
 
 DATA_DIR = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'data')
 
@@ -77,7 +76,7 @@ class IOHandlerTest(unittest.TestCase):
             self.assertEqual('file', urlparse(self.iohandler.url).scheme)
 
         if self.iohandler.source_type == SOURCE_TYPE.STREAM:
-            source = StringIO(text_type(self._value))
+            source = StringIO(str(self._value))
             self.iohandler.stream = source
 
         file_path = self.iohandler.file
@@ -87,16 +86,13 @@ class IOHandlerTest(unittest.TestCase):
         file_handler.close()
 
         if self.iohandler.source_type == SOURCE_TYPE.STREAM:
-            source = StringIO(text_type(self._value))
+            source = StringIO(str(self._value))
             self.iohandler.stream = source
 
         stream_val = self.iohandler.stream.read()
         self.iohandler.stream.close()
 
-        if PY2 and isinstance(stream_val, str):
-            self.assertEqual(self._value, stream_val.decode('utf-8'),
-                             'Stream obtained')
-        elif not PY2 and isinstance(stream_val, bytes):
+        if isinstance(stream_val, bytes):
             self.assertEqual(self._value, stream_val.decode('utf-8'),
                              'Stream obtained')
         else:
@@ -104,7 +100,7 @@ class IOHandlerTest(unittest.TestCase):
                              'Stream obtained')
 
         if self.iohandler.source_type == SOURCE_TYPE.STREAM:
-            source = StringIO(text_type(self._value))
+            source = StringIO(str(self._value))
             self.iohandler.stream = source
 
         # self.assertEqual(stream_val, self.iohandler.memory_object,
@@ -112,15 +108,13 @@ class IOHandlerTest(unittest.TestCase):
 
     def test_data(self):
         """Test data input IOHandler"""
-        if PY2:
-            self.skipTest('fails on python 2.7')
         self.iohandler.data = self._value
         self.iohandler.data_format = Format('foo', extension='.foo')
         self._test_outout(SOURCE_TYPE.DATA, '.foo')
 
     def test_stream(self):
         """Test stream input IOHandler"""
-        source = StringIO(text_type(self._value))
+        source = StringIO(str(self._value))
         self.iohandler.stream = source
         self._test_outout(SOURCE_TYPE.STREAM)
 
@@ -163,8 +157,6 @@ class IOHandlerTest(unittest.TestCase):
         self.skipTest('Memory object not implemented')
 
     def test_data_bytes(self):
-        if PY2:
-            self.skipTest('fails on python 2.7')
         self._value = b'aa'
 
         self.iohandler.data = self._value
@@ -266,15 +258,6 @@ class SerializationComplexInputTest(unittest.TestCase):
         self.assertEqual(complex_1.as_reference, complex_2.as_reference)
         self.assertEqual(complex_1.translations, complex_2.translations)
 
-        self.assertEqual(complex_1.prop, complex_2.prop)
-
-        if complex_1.prop != 'url':
-            # don't download the file when running tests
-            self.assertEqual(complex_1.file, complex_2.file)
-            self.assertEqual(complex_1.data, complex_2.data)
-
-        self.assertEqual(complex_1.url, complex_2.url)
-
     def test_complex_input_file(self):
         complex = self.make_complex_input()
         some_file = os.path.join(self.tmp_dir, "some_file.txt")
@@ -284,6 +267,9 @@ class SerializationComplexInputTest(unittest.TestCase):
         complex2 = inout.inputs.ComplexInput.from_json(complex.json)
         self.assert_complex_equals(complex, complex2)
         self.assertEqual(complex.prop, 'file')
+        self.assertEqual(complex2.prop, 'file')
+        self.assertEqual(complex.file, complex2.file)
+        self.assertEqual(complex.data, complex2.data)
 
     def test_complex_input_data(self):
         complex = self.make_complex_input()
@@ -292,11 +278,11 @@ class SerializationComplexInputTest(unittest.TestCase):
         assert complex.json['data'] == '<![CDATA[some data]]>'
         # dump to json and load it again
         complex2 = inout.inputs.ComplexInput.from_json(complex.json)
-        # it's expected that the file path changed
-        complex._file = complex2.file
 
         self.assert_complex_equals(complex, complex2)
         self.assertEqual(complex.prop, 'data')
+        self.assertEqual(complex2.prop, 'data')
+        self.assertEqual(complex.data, complex2.data)
 
     def test_complex_input_stream(self):
         complex = self.make_complex_input()
@@ -306,12 +292,10 @@ class SerializationComplexInputTest(unittest.TestCase):
         # dump to json and load it again
         complex2 = inout.inputs.ComplexInput.from_json(complex.json)
         # the serialized stream becomes a data type
-        # we hard-code it for the testing comparison
-        complex.prop = 'data'
-        # it's expected that the file path changed
-        complex._file = complex2.file
-
+        self.assertEqual(complex.prop, 'stream')
+        self.assertEqual(complex2.prop, 'data')
         self.assert_complex_equals(complex, complex2)
+        self.assertEqual(complex.data, complex2.data)
 
     def test_complex_input_url(self):
         complex = self.make_complex_input()
@@ -468,7 +452,7 @@ class ComplexOutputTest(unittest.TestCase):
             supported_formats=[data_format],
             mode=MODE.NONE,
             translations={"fr-CA": {"title": "Mon output", "abstract": "Une description"}},
-            )
+        )
 
         self.complex_out_nc = inout.outputs.ComplexOutput(
             identifier="netcdf",
@@ -478,7 +462,7 @@ class ComplexOutputTest(unittest.TestCase):
             supported_formats=[get_data_format('application/x-netcdf')],
             mode=MODE.NONE)
 
-        self.data = json.dumps({'a': 1, 'unicodé': u'éîïç', })
+        self.data = json.dumps({'a': 1, 'unicodé': 'éîïç', })
         self.ncfile = os.path.join(DATA_DIR, 'netcdf', 'time.nc')
 
         self.test_fn = os.path.join(self.complex_out.workdir, 'test.json')
@@ -508,24 +492,18 @@ class ComplexOutputTest(unittest.TestCase):
     def test_file_handler(self):
         self.complex_out.file = self.test_fn
         self.assertEqual(self.complex_out.data, self.data)
-        if PY2:
-            self.assertEqual(self.complex_out.stream.read(), self.data)
-        else:
-            with self.complex_out.stream as s:
-                self.assertEqual(s.read(), bytes(self.data, encoding='utf8'))
+        with self.complex_out.stream as s:
+            self.assertEqual(s.read(), bytes(self.data, encoding='utf8'))
 
-        with open(urlparse(self.complex_out.url).path) as f:
+        path = uri_to_path(self.complex_out.url)
+        with open(path) as f:
             self.assertEqual(f.read(), self.data)
 
     def test_file_handler_netcdf(self):
-        if PY2:
-            self.skipTest('fails on python 2.7')
         self.complex_out_nc.file = self.ncfile
         self.complex_out_nc.base64
 
     def test_data_handler(self):
-        if PY2:
-            self.skipTest('fails on python 2.7')
         self.complex_out.data = self.data
         with open(self.complex_out.file) as f:
             self.assertEqual(f.read(), self.data)
@@ -533,10 +511,7 @@ class ComplexOutputTest(unittest.TestCase):
     def test_base64(self):
         self.complex_out.data = self.data
         b = self.complex_out.base64
-        if PY2:
-            self.assertEqual(base64.b64decode(b), self.data)
-        else:
-            self.assertEqual(base64.b64decode(b).decode(), self.data)
+        self.assertEqual(base64.b64decode(b).decode(), self.data)
 
     def test_url_handler(self):
         wfsResource = 'http://demo.mapserver.org/cgi-bin/wfs?' \
@@ -586,7 +561,7 @@ class LiteralInputTest(unittest.TestCase):
             mode=2,
             allowed_values=(1, 2, (3, 3, 12)),
             default=6,
-            uoms=(UOM("metre"),),
+            uoms=(UOM("metre"), UOM("km / h", "custom reference")),
             translations={"fr-CA": {"title": "Mon input", "abstract": "Une description"}},
         )
 
@@ -620,8 +595,10 @@ class LiteralInputTest(unittest.TestCase):
         self.literal_input.data = 9
         out = self.literal_input.json
 
-        self.assertTrue('uoms' in out, 'UOMs does not exist')
-        self.assertTrue('uom' in out, 'uom does not exist')
+        self.assertEqual(out['uoms'][0]["uom"], "metre")
+        self.assertEqual(out['uoms'][1]["uom"], "km / h")
+        self.assertEqual(out['uoms'][1]["reference"], "custom reference")
+        self.assertEqual(out['uom']["uom"], "metre")
         self.assertFalse(out['workdir'], 'Workdir exist')
         self.assertEqual(out['data_type'], 'integer', 'Data type is integer')
         self.assertFalse(out['abstract'], 'abstract exist')
@@ -677,6 +654,7 @@ class LiteralInputTest(unittest.TestCase):
         identifier = get_translation(self.literal_input, "identifier", "fr-CA")
         assert identifier == self.literal_input.identifier
 
+
 class LiteralOutputTest(unittest.TestCase):
     """LiteralOutput test cases"""
 
@@ -686,6 +664,7 @@ class LiteralOutputTest(unittest.TestCase):
             "literaloutput",
             data_type="integer",
             title="Literal Output",
+            uoms=[UOM("km / h", "custom reference"), UOM("m / s", "other reference")],
             translations={"fr-CA": {"title": "Mon output", "abstract": "Une description"}},
         )
 
@@ -702,6 +681,8 @@ class LiteralOutputTest(unittest.TestCase):
     def test_json(self):
         new_output = inout.outputs.LiteralOutput.from_json(self.literal_output.json)
         self.assertEqual(new_output.identifier, 'literaloutput')
+        self.assertEqual(new_output.uom, self.literal_output.uom)
+        self.assertEqual(new_output.uoms, self.literal_output.uoms)
         self.assertEqual(
             new_output.translations,
             {"fr-ca": {"title": "Mon output", "abstract": "Une description"}},
@@ -783,6 +764,12 @@ class TestMetaLink(unittest.TestCase):
         mf._set_workdir(self.tmp_dir)
         return mf
 
+    def metafile_with_url(self):
+        mf = MetaFile('identifier', 'title', fmt=FORMATS.JSON)
+        mf.url = "https://pywps.org/"
+        mf._set_workdir(self.tmp_dir)
+        return mf
+
     def test_metafile(self):
         mf = self.metafile()
         self.assertEqual('identifier', mf.identity)
@@ -832,6 +819,25 @@ class TestMetaLink(unittest.TestCase):
         ml4.checksums = True
         assert 'hash' in ml4.xml
 
+    def test_size(self):
+        ml4 = self.metalink4()
+        ml4.append(self.metafile_with_url())
+        assert 'size' in ml4.xml
+
+    def test_no_size(self):
+        ml4 = self.metalink4()
+        mf = self.metafile_with_url()
+        mf.size = 0
+        ml4.files = [mf]
+        assert 'size' not in ml4.xml
+
+    def test_set_size(self):
+        ml4 = self.metalink4()
+        mf = self.metafile_with_url()
+        mf.size = 100
+        ml4.files = [mf]
+        assert '<size>100</size>' in ml4.xml
+
 
 def load_tests(loader=None, tests=None, pattern=None):
     if not loader:
